package info.frostfs.sdk.services.impl;

import frostfs.accounting.AccountingServiceGrpc;
import frostfs.accounting.Service;
import frostfs.accounting.Types;
import info.frostfs.sdk.jdo.ClientEnvironment;
import info.frostfs.sdk.jdo.parameters.CallContext;
import info.frostfs.sdk.mappers.object.OwnerIdMapper;
import info.frostfs.sdk.services.AccountingClient;
import info.frostfs.sdk.services.ContextAccessor;
import info.frostfs.sdk.tools.RequestConstructor;
import info.frostfs.sdk.tools.RequestSigner;
import info.frostfs.sdk.tools.Verifier;

import static info.frostfs.sdk.utils.DeadLineUtil.deadLineAfter;

public class AccountingClientImpl extends ContextAccessor implements AccountingClient {
    private final AccountingServiceGrpc.AccountingServiceBlockingStub serviceBlockingStub;

    public AccountingClientImpl(ClientEnvironment clientEnvironment) {
        super(clientEnvironment);
        this.serviceBlockingStub = AccountingServiceGrpc.newBlockingStub(getContext().getChannel());
    }

    @Override
    public Types.Decimal getBalance(CallContext ctx) {
        var request = createGetRequest();

        var service = deadLineAfter(serviceBlockingStub, ctx.getTimeout(), ctx.getTimeUnit());
        var response = service.balance(request);

        Verifier.checkResponse(response);
        return response.getBody().getBalance();
    }

    private Service.BalanceRequest createGetRequest() {
        var body = Service.BalanceRequest.Body.newBuilder()
                .setOwnerId(OwnerIdMapper.toGrpcMessage(getContext().getOwnerId()))
                .build();
        var request = Service.BalanceRequest.newBuilder()
                .setBody(body);

        RequestConstructor.addMetaHeader(request);
        RequestSigner.sign(request, getContext().getKey());

        return request.build();
    }
}
