package info.frostfs.sdk.services.impl.interceptor;

import io.grpc.Channel;
import io.grpc.MethodDescriptor;
import io.grpc.MethodDescriptor.MethodType;

public class GrpcMethod {
    private final String targetName;
    private final String serviceName;
    private final String methodName;
    private final MethodType type;

    private GrpcMethod(String targetName, String serviceName, String methodName, MethodType type) {
        this.targetName = targetName;
        this.serviceName = serviceName;
        this.methodName = methodName;
        this.type = type;
    }

    static GrpcMethod of(MethodDescriptor<?, ?> method, Channel channel) {
        String serviceName = MethodDescriptor.extractFullServiceName(method.getFullMethodName());

        // Full method names are of the form: "full.serviceName/MethodName". We extract the last part.
        String methodName = method.getFullMethodName().substring(serviceName.length() + 1);
        return new GrpcMethod(channel.authority(), serviceName, methodName, method.getType());
    }

    String targetName() {
        return targetName;
    }

    String serviceName() {
        return serviceName;
    }

    String methodName() {
        return methodName;
    }

    String type() {
        return type.toString();
    }

    boolean streamsRequests() {
        return type == MethodType.CLIENT_STREAMING || type == MethodType.BIDI_STREAMING;
    }

    boolean streamsResponses() {
        return type == MethodType.SERVER_STREAMING || type == MethodType.BIDI_STREAMING;
    }
}
