package info.frostfs.sdk.services.impl.rwhelper;

import frostfs.object.Service;
import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectReader;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import info.frostfs.sdk.tools.Verifier;

import java.util.Iterator;

import static info.frostfs.sdk.constants.ErrorConst.UNEXPECTED_MESSAGE_TYPE_TEMPLATE;
import static info.frostfs.sdk.constants.ErrorConst.UNEXPECTED_STREAM;

public class ObjectReaderImpl implements ObjectReader {
    public Iterator<Service.GetResponse> call;

    public ObjectReaderImpl(Iterator<Service.GetResponse> call) {
        this.call = call;
    }

    public Types.Object readHeader() {
        if (!call.hasNext()) {
            throw new ProcessFrostFSException(UNEXPECTED_STREAM);
        }

        var response = call.next();
        Verifier.checkResponse(response);

        if (response.getBody().getObjectPartCase().getNumber() != Service.GetResponse.Body.INIT_FIELD_NUMBER) {
            throw new ProcessFrostFSException(
                    String.format(
                            UNEXPECTED_MESSAGE_TYPE_TEMPLATE,
                            Service.GetResponse.Body.INIT_FIELD_NUMBER,
                            response.getBody().getObjectPartCase().getNumber()
                    )
            );
        }

        return Types.Object.newBuilder()
                .setObjectId(response.getBody().getInit().getObjectId())
                .setHeader(response.getBody().getInit().getHeader())
                .build();
    }

    public byte[] readChunk() {
        if (!call.hasNext()) {
            return null;
        }

        var response = call.next();
        Verifier.checkResponse(response);

        if (response.getBody().getObjectPartCase().getNumber() != Service.GetResponse.Body.CHUNK_FIELD_NUMBER) {
            throw new ProcessFrostFSException(
                    String.format(
                            UNEXPECTED_MESSAGE_TYPE_TEMPLATE,
                            Service.GetResponse.Body.CHUNK_FIELD_NUMBER,
                            response.getBody().getObjectPartCase().getNumber()
                    )
            );
        }

        return response.getBody().getChunk().toByteArray();
    }
}
