package info.frostfs.sdk.services.impl.rwhelper;

import frostfs.object.ObjectServiceGrpc;
import frostfs.object.Service;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import info.frostfs.sdk.utils.WaitUtil;
import io.grpc.stub.StreamObserver;
import lombok.Getter;

import static info.frostfs.sdk.constants.AppConst.DEFAULT_POLL_INTERVAL;
import static info.frostfs.sdk.constants.ErrorConst.PROTO_MESSAGE_IS_EMPTY_TEMPLATE;
import static java.util.Objects.isNull;

public class ObjectStreamer {
    private final StreamObserver<Service.PutRequest> requestObserver;
    private final PutResponseCallback responseObserver;

    public ObjectStreamer(ObjectServiceGrpc.ObjectServiceStub objectServiceStub) {
        PutResponseCallback responseObserver = new PutResponseCallback();

        this.responseObserver = responseObserver;
        this.requestObserver = objectServiceStub.put(responseObserver);
    }

    public void write(Service.PutRequest request) {
        if (isNull(request)) {
            throw new ProcessFrostFSException(
                    String.format(PROTO_MESSAGE_IS_EMPTY_TEMPLATE, Service.PutRequest.class.getName())
            );
        }
        requestObserver.onNext(request);
    }

    public Service.PutResponse complete() {
        requestObserver.onCompleted();

        while (isNull(responseObserver.getResponse())) {
            WaitUtil.sleep(DEFAULT_POLL_INTERVAL);
        }

        return responseObserver.getResponse();
    }

    @Getter
    private static class PutResponseCallback implements StreamObserver<Service.PutResponse> {
        private Service.PutResponse response;

        @Override
        public void onNext(Service.PutResponse putResponse) {
            this.response = putResponse;
        }

        @Override
        public void onError(Throwable throwable) {
            throw new ProcessFrostFSException(throwable);
        }

        @Override
        public void onCompleted() {
        }
    }
}
