package info.frostfs.sdk.services.impl.rwhelper;

import frostfs.object.ObjectServiceGrpc;
import frostfs.object.Service;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import info.frostfs.sdk.utils.WaitUtil;
import io.grpc.stub.StreamObserver;
import lombok.Getter;

import static info.frostfs.sdk.constants.AppConst.DEFAULT_POLL_INTERVAL;
import static info.frostfs.sdk.constants.ErrorConst.PROTO_MESSAGE_IS_EMPTY_TEMPLATE;
import static java.util.Objects.isNull;

public class PatchStreamer {
    private final StreamObserver<Service.PatchRequest> requestObserver;
    private final PatchResponseCallback responseObserver;

    public PatchStreamer(ObjectServiceGrpc.ObjectServiceStub objectServiceStub) {
        PatchResponseCallback responseObserver = new PatchResponseCallback();

        this.responseObserver = responseObserver;
        this.requestObserver = objectServiceStub.patch(responseObserver);
    }

    public void write(Service.PatchRequest request) {
        if (isNull(request)) {
            throw new ProcessFrostFSException(
                    String.format(PROTO_MESSAGE_IS_EMPTY_TEMPLATE, Service.PutRequest.class.getName())
            );
        }
        requestObserver.onNext(request);
    }

    public Service.PatchResponse complete() {
        requestObserver.onCompleted();

        while (isNull(responseObserver.getResponse())) {
            WaitUtil.sleep(DEFAULT_POLL_INTERVAL);
        }

        return responseObserver.getResponse();
    }

    @Getter
    private static class PatchResponseCallback implements StreamObserver<Service.PatchResponse> {
        private Service.PatchResponse response;

        @Override
        public void onNext(Service.PatchResponse patchResponse) {
            this.response = patchResponse;
        }

        @Override
        public void onError(Throwable throwable) {
            throw new ProcessFrostFSException(throwable);
        }

        @Override
        public void onCompleted() {
        }
    }
}
