package info.frostfs.sdk.tools;

import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import info.frostfs.sdk.jdo.ClientSettings;
import info.frostfs.sdk.utils.Validator;
import io.grpc.ManagedChannel;
import io.grpc.netty.NettyChannelBuilder;

import java.net.URI;
import java.net.URISyntaxException;

import static info.frostfs.sdk.constants.ErrorConst.INVALID_HOST_TEMPLATE;
import static java.util.Objects.isNull;

public class GrpcClient {
    private GrpcClient() {
    }

    public static ManagedChannel initGrpcChannel(ClientSettings clientSettings) {
        Validator.validate(clientSettings);

        try {
            URI uri = new URI(clientSettings.getHost());
            var channelBuilder = isNull(clientSettings.getCredentials())
                    ? NettyChannelBuilder.forAddress(uri.getHost(), uri.getPort()).usePlaintext()
                    : NettyChannelBuilder.forAddress(uri.getHost(), uri.getPort(), clientSettings.getCredentials());

            return channelBuilder.build();
        } catch (URISyntaxException exp) {
            throw new ValidationFrostFSException(
                    String.format(INVALID_HOST_TEMPLATE, clientSettings.getHost(), exp.getMessage())
            );
        }
    }

    public static ManagedChannel initGrpcChannel(String address) {
        try {
            URI uri = new URI(address);
            return NettyChannelBuilder.forAddress(uri.getHost(), uri.getPort()).usePlaintext().build();
        } catch (URISyntaxException exp) {
            throw new ValidationFrostFSException(
                    String.format(INVALID_HOST_TEMPLATE, address, exp.getMessage())
            );
        }
    }
}
