package info.frostfs.sdk.utils;

import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import io.grpc.stub.AbstractStub;

import java.util.concurrent.TimeUnit;

import static info.frostfs.sdk.constants.ErrorConst.PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

public class DeadLineUtil {
    private DeadLineUtil() {
    }

    public static <T extends AbstractStub<T>> T deadLineAfter(T stub, long deadLine, TimeUnit timeUnit) {
        if (isNull(stub)) {
            throw new ValidationFrostFSException(
                    String.format(PARAM_IS_MISSING_TEMPLATE, AbstractStub.class.getName())
            );
        }

        timeUnit = isNull(timeUnit) ? TimeUnit.MILLISECONDS : timeUnit;
        return deadLine > 0 ? stub.withDeadlineAfter(deadLine, timeUnit) : stub;
    }
}
