package info.frostfs.sdk.testgenerator;

import com.google.protobuf.ByteString;
import frostfs.refs.Types;

import java.util.Arrays;
import java.util.Random;

public class RefsGenerator {

    public static Types.Version generateVersion() {
        return Types.Version.newBuilder()
                .setMajor(2)
                .setMinor(1)
                .build();
    }

    public static Types.OwnerID generateOwnerID() {
        return Types.OwnerID.newBuilder()
                .setValue(ByteString.copyFrom(new byte[]{1, 2, 3}))
                .build();
    }

    public static Types.Address generateAddress() {
        return Types.Address.newBuilder()
                .setObjectId(generateObjectID())
                .setContainerId(generateContainerID())
                .build();
    }

    public static Types.ObjectID generateObjectID() {
        return Types.ObjectID.newBuilder()
                .setValue(ByteString.copyFrom(new byte[]{1, 2, 3}))
                .build();
    }

    public static Iterable<Types.ObjectID> generateObjectIDs() {
        return Arrays.asList(generateObjectID(), generateObjectID());
    }

    public static Types.ContainerID generateContainerID() {
        return Types.ContainerID.newBuilder()
                .setValue(ByteString.copyFrom(new byte[]{1, 2, 3}))
                .build();
    }

    public static Iterable<Types.ContainerID> generateContainerIDs() {
        return Arrays.asList(generateContainerID(), generateContainerID());
    }

    public static Types.Signature generateSignature() {
        return Types.Signature.newBuilder()
                .setKey(ByteString.copyFrom(new byte[]{1}))
                .setSign(ByteString.copyFrom(new byte[]{2}))
                .setScheme(Types.SignatureScheme.forNumber(new Random().nextInt(3)))
                .build();
    }

    public static Types.Checksum generateChecksum() {
        return Types.Checksum.newBuilder()
                .setType(Types.ChecksumType.SHA256)
                .setSum(ByteString.copyFrom(new byte[]{1, 2, 3}))
                .build();
    }
}
