package info.frostfs.sdk.testgenerator;

import com.google.protobuf.ByteString;
import frostfs.session.Service;
import frostfs.session.Types;

import java.util.Arrays;
import java.util.Random;

public class SessionGenerator {

    public static Service.CreateRequest.Body generateCreateRequestBody() {
        return Service.CreateRequest.Body.newBuilder()
                .setExpiration(555)
                .setOwnerId(RefsGenerator.generateOwnerID())
                .build();
    }

    public static Service.CreateRequest generateCreateRequest() {
        return Service.CreateRequest.newBuilder()
                .setBody(generateCreateRequestBody())
                .setMetaHeader(generateRequestMetaHeader())
                .setVerifyHeader(generateRequestVerificationHeader())
                .build();
    }

    public static Service.CreateResponse.Body generateCreateResponseBody() {
        return Service.CreateResponse.Body.newBuilder()
                .setId(ByteString.copyFrom(new byte[]{1, 2, 3}))
                .setSessionKey(ByteString.copyFrom(new byte[]{4, 5, 6}))
                .build();
    }

    public static Service.CreateResponse generateCreateResponse() {
        return Service.CreateResponse.newBuilder()
                .setBody(generateCreateResponseBody())
                .setMetaHeader(generateResponseMetaHeader())
                .setVerifyHeader(generateResponseVerificationHeader())
                .build();
    }

    public static Types.ResponseVerificationHeader generateResponseVerificationHeader() {
        return generateResponseVerificationHeader(true);
    }

    public static Types.ResponseVerificationHeader generateResponseVerificationHeader(boolean withOrigin) {
        var builder = Types.ResponseVerificationHeader.newBuilder()
                .setBodySignature(RefsGenerator.generateSignature())
                .setMetaSignature(RefsGenerator.generateSignature())
                .setOriginSignature(RefsGenerator.generateSignature());

        if (withOrigin) {
            builder.setOrigin(generateResponseVerificationHeader(false));
        }

        return builder.build();
    }

    public static Types.ResponseMetaHeader generateResponseMetaHeader() {
        return generateResponseMetaHeader(true);
    }

    public static Types.ResponseMetaHeader generateResponseMetaHeader(boolean withOrigin) {
        var builder = Types.ResponseMetaHeader.newBuilder()
                .setEpoch(13)
                .setTtl(100)
                .addAllXHeaders(generateXHeaders())
                .setVersion(RefsGenerator.generateVersion())
                .setStatus(StatusGenerator.generateStatus());

        if (withOrigin) {
            builder.setOrigin(generateResponseMetaHeader(false));
        }

        return builder.build();
    }

    public static Types.RequestVerificationHeader generateRequestVerificationHeader() {
        return generateRequestVerificationHeader(true);
    }

    public static Types.RequestVerificationHeader generateRequestVerificationHeader(boolean withOrigin) {
        var builder = Types.RequestVerificationHeader.newBuilder()
                .setBodySignature(RefsGenerator.generateSignature())
                .setMetaSignature(RefsGenerator.generateSignature())
                .setOriginSignature(RefsGenerator.generateSignature());

        if (withOrigin) {
            builder.setOrigin(generateRequestVerificationHeader(false));
        }

        return builder.build();
    }

    public static Types.RequestMetaHeader generateRequestMetaHeader() {
        return generateRequestMetaHeader(true);
    }

    public static Types.RequestMetaHeader generateRequestMetaHeader(boolean withOrigin) {
        var builder = Types.RequestMetaHeader.newBuilder()
                .setEpoch(13)
                .setTtl(100)
                .setMagicNumber(1337)
                .addAllXHeaders(generateXHeaders())
                .setVersion(RefsGenerator.generateVersion())
                .setSessionToken(generateSessionToken());

        if (withOrigin) {
            builder.setOrigin(generateRequestMetaHeader(false));
        }

        return builder.build();
    }

    public static Types.SessionToken.Body.TokenLifetime generateTokenLifeTime() {
        return Types.SessionToken.Body.TokenLifetime.newBuilder()
                .setExp(1)
                .setIat(2)
                .setNbf(3)
                .build();
    }

    public static Types.SessionToken.Body generateSessionTokenBody() {
        var builder = Types.SessionToken.Body.newBuilder()
                .setId(ByteString.copyFrom(new byte[]{1}))
                .setSessionKey(ByteString.copyFrom(new byte[]{2}))
                .setOwnerId(RefsGenerator.generateOwnerID())
                .setLifetime(generateTokenLifeTime());

        if (new Random().nextBoolean()) {
            builder.setContainer(generateContainerSessionContext());
        } else {
            builder.setObject(generateObjectSessionContext());
        }

        return builder.build();
    }

    public static Types.SessionToken generateSessionToken() {
        return Types.SessionToken.newBuilder()
                .setBody(generateSessionTokenBody())
                .setSignature(RefsGenerator.generateSignature())
                .build();
    }

    public static Types.ObjectSessionContext.Target generateObjectSessionContextTarget() {
        return Types.ObjectSessionContext.Target.newBuilder()
                .setContainer(RefsGenerator.generateContainerID())
                .addObjects(RefsGenerator.generateObjectID())
                .build();
    }

    public static Types.ObjectSessionContext generateObjectSessionContext() {
        return Types.ObjectSessionContext.newBuilder()
                .setVerb(Types.ObjectSessionContext.Verb.HEAD)
                .setTarget(generateObjectSessionContextTarget())
                .build();
    }

    public static Types.ContainerSessionContext generateContainerSessionContext() {
        return Types.ContainerSessionContext.newBuilder()
                .setVerb(Types.ContainerSessionContext.Verb.DELETE)
                .setWildcard(true)
                .setContainerId(RefsGenerator.generateContainerID())
                .build();
    }

    public static Types.XHeader generateXHeader() {
        return Types.XHeader.newBuilder()
                .setKey("key")
                .setValue("val")
                .build();
    }

    public static Iterable<Types.XHeader> generateXHeaders() {
        return Arrays.asList(generateXHeader(), generateXHeader());
    }
}
