package info.frostfs.sdk.dto.container;

import info.frostfs.sdk.Base58;
import info.frostfs.sdk.constants.AppConst;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_NOT_SHA256;
import static java.util.Objects.isNull;

@Getter
public class ContainerId {

    private final String value;

    public ContainerId(String value) {
        if (StringUtils.isEmpty(value)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, ContainerId.class.getName())
            );
        }

        this.value = value;
    }

    public ContainerId(byte[] hash) {
        if (isNull(hash) || hash.length != AppConst.SHA256_HASH_LENGTH) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_NOT_SHA256, ContainerId.class.getName())
            );
        }

        this.value = Base58.encode(hash);
    }

    @Override
    public String toString() {
        return value;
    }

    public byte[] toHash() {
        return Base58.decode(value);
    }
}
