package info.frostfs.sdk.dto.object;

import info.frostfs.sdk.dto.container.ContainerId;

import java.security.MessageDigest;

import static info.frostfs.sdk.Helper.getSha256Instance;
import static java.util.Objects.isNull;

public class LargeObject extends ObjectFrostFS {
    private final MessageDigest payloadHash;

    public LargeObject(ContainerId cid) {
        super(cid, new byte[]{});
        this.payloadHash = getSha256Instance();
    }

    public void appendBlock(byte[] bytes, int count) {
        if (count == 0 || isNull(bytes) || bytes.length == 0) {
            return;
        }

        this.getHeader().increasePayloadLength(count);
        this.payloadHash.update(bytes, 0, count);
    }

    public void calculateHash() {
        this.getHeader().setPayloadCheckSum(this.payloadHash.digest());
    }

    public long getPayloadLength() {
        return getHeader().getPayloadLength();
    }
}
