package info.frostfs.sdk.dto.object;

import info.frostfs.sdk.constants.FieldConst;
import info.frostfs.sdk.constants.FilterConst;
import info.frostfs.sdk.dto.CheckSum;
import info.frostfs.sdk.dto.container.ContainerId;
import info.frostfs.sdk.dto.netmap.Version;
import info.frostfs.sdk.enums.ObjectMatchType;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
@AllArgsConstructor
public abstract class ObjectFilter<T> {
    private ObjectMatchType matchType;
    private String key;
    private T value;

    public String getSerializedValue() {
        return value.toString();
    }

    public static class FilterByAttribute extends ObjectFilter<String> {
        public FilterByAttribute(ObjectMatchType matchType, String key, String value) {
            super(matchType, key, value);
        }
    }

    public static class FilterByObjectId extends ObjectFilter<ObjectId> {
        public FilterByObjectId(ObjectMatchType matchType, ObjectId value) {
            super(matchType, FilterConst.FILTER_HEADER_OBJECT_ID, value);
        }
    }

    public static class FilterByOwnerId extends ObjectFilter<OwnerId> {
        public FilterByOwnerId(ObjectMatchType matchType, OwnerId value) {
            super(matchType, FilterConst.FILTER_HEADER_OWNER_ID, value);
        }
    }

    public static class FilterByVersion extends ObjectFilter<Version> {
        public FilterByVersion(ObjectMatchType matchType, Version value) {
            super(matchType, FilterConst.FILTER_HEADER_VERSION, value);
        }
    }

    public static class FilterByContainerId extends ObjectFilter<ContainerId> {
        public FilterByContainerId(ObjectMatchType matchType, ContainerId value) {
            super(matchType, FilterConst.FILTER_HEADER_CONTAINER_ID, value);
        }
    }

    public static class FilterByEpoch extends ObjectFilter<Long> {
        public FilterByEpoch(ObjectMatchType matchType, long value) {
            super(matchType, FilterConst.FILTER_HEADER_CREATION_EPOCH, value);
        }
    }

    public static class FilterByPayloadLength extends ObjectFilter<Long> {
        public FilterByPayloadLength(ObjectMatchType matchType, long value) {
            super(matchType, FilterConst.FILTER_HEADER_PAYLOAD_LENGTH, value);
        }
    }

    public static class FilterByPayloadHash extends ObjectFilter<CheckSum> {
        public FilterByPayloadHash(ObjectMatchType matchType, CheckSum value) {
            super(matchType, FilterConst.FILTER_HEADER_PAYLOAD_HASH, value);
        }
    }

    public static class FilterByParent extends ObjectFilter<ObjectId> {
        public FilterByParent(ObjectMatchType matchType, ObjectId value) {
            super(matchType, FilterConst.FILTER_HEADER_PARENT, value);
        }
    }

    public static class FilterBySplitId extends ObjectFilter<SplitId> {
        public FilterBySplitId(ObjectMatchType matchType, SplitId value) {
            super(matchType, FilterConst.FILTER_HEADER_SPLIT_ID, value);
        }
    }


    public static class FilterByECParent extends ObjectFilter<ObjectId> {
        public FilterByECParent(ObjectMatchType matchType, ObjectId value) {
            super(matchType, FilterConst.FILTER_HEADER_EC_PARENT, value);
        }
    }

    public static class FilterByRootObject extends ObjectFilter<String> {
        public FilterByRootObject() {
            super(ObjectMatchType.UNSPECIFIED, FilterConst.FILTER_HEADER_ROOT, FieldConst.EMPTY_STRING);
        }
    }

    public static class FilterByPhysicallyStored extends ObjectFilter<String> {
        public FilterByPhysicallyStored() {
            super(ObjectMatchType.UNSPECIFIED, FilterConst.FILTER_HEADER_PHY, FieldConst.EMPTY_STRING);
        }
    }
}
