package info.frostfs.sdk.dto.object;

import info.frostfs.sdk.Base58;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import static info.frostfs.sdk.KeyExtension.publicKeyToAddress;
import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING;
import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

@Getter
public class OwnerId {
    private final String value;

    public OwnerId(String value) {
        if (StringUtils.isBlank(value)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, OwnerId.class.getName())
            );
        }

        this.value = value;
    }

    public OwnerId(byte[] publicKey) {
        if (isNull(publicKey) || publicKey.length == 0) {
            throw new ValidationFrostFSException(INPUT_PARAM_IS_MISSING);
        }

        this.value = publicKeyToAddress(publicKey);
    }

    public byte[] toHash() {
        return Base58.decode(value);
    }

    @Override
    public String toString() {
        return value;
    }
}
