package info.frostfs.sdk.dto.response;

import info.frostfs.sdk.dto.netmap.Version;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import lombok.Getter;
import lombok.experimental.FieldNameConstants;

import static info.frostfs.sdk.constants.AppConst.DEFAULT_MAJOR_VERSION;
import static info.frostfs.sdk.constants.AppConst.DEFAULT_MINOR_VERSION;
import static info.frostfs.sdk.constants.ErrorConst.*;
import static java.util.Objects.isNull;

@Getter
@FieldNameConstants
public class MetaHeader {
    private Version version;
    private int epoch;
    private int ttl;

    public MetaHeader(Version version, int epoch, int ttl) {
        setVersion(version);
        setEpoch(epoch);
        setTtl(ttl);
    }

    public MetaHeader() {
        this.version = new Version(DEFAULT_MAJOR_VERSION, DEFAULT_MINOR_VERSION);
        this.epoch = 0;
        this.ttl = 2;
    }

    public void setVersion(Version version) {
        if (isNull(version)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Version.class.getName())
            );
        }

        this.version = version;
    }

    public void setEpoch(int epoch) {
        if (epoch < 0) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_GREATER_OR_EQUAL_ZERO, Fields.epoch)
            );
        }

        this.epoch = epoch;
    }

    public void setTtl(int ttl) {
        if (ttl <= 0) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_GREATER_ZERO, Fields.ttl)
            );
        }

        this.ttl = ttl;
    }
}
