package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Service;
import frostfs.netmap.Types;
import frostfs.netmap.Types.NodeInfo.Attribute;
import info.frostfs.sdk.dto.netmap.NodeInfo;
import info.frostfs.sdk.enums.NodeState;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;

import java.util.stream.Collectors;

import static info.frostfs.sdk.constants.ErrorConst.UNKNOWN_ENUM_VALUE_TEMPLATE;
import static java.util.Objects.isNull;

public class NodeInfoMapper {
    private NodeInfoMapper() {
    }

    public static NodeInfo toModel(Service.LocalNodeInfoResponse.Body nodeInfo) {
        if (isNull(nodeInfo) || nodeInfo.getSerializedSize() == 0) {
            return null;
        }

        return toModel(nodeInfo.getNodeInfo(), nodeInfo.getVersion());
    }

    public static NodeInfo toModel(Types.NodeInfo nodeInfo, frostfs.refs.Types.Version version) {
        if (isNull(nodeInfo) || nodeInfo.getSerializedSize() == 0) {
            return null;
        }

        NodeState nodeState = NodeState.get(nodeInfo.getState().getNumber());
        if (isNull(nodeState)) {
            throw new ProcessFrostFSException(
                    String.format(UNKNOWN_ENUM_VALUE_TEMPLATE, NodeState.class.getName(), nodeInfo.getState())
            );
        }

        return new NodeInfo(
                nodeState,
                VersionMapper.toModel(version),
                nodeInfo.getAddressesList(),
                nodeInfo.getAttributesList().stream().collect(Collectors.toMap(Attribute::getKey, Attribute::getValue)),
                nodeInfo.getPublicKey().toByteArray()
        );
    }
}
