package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Types;
import info.frostfs.sdk.dto.netmap.PlacementPolicy;
import info.frostfs.sdk.dto.netmap.Replica;

import static java.util.Objects.isNull;

public class PlacementPolicyMapper {
    private PlacementPolicyMapper() {
    }

    public static Types.PlacementPolicy toGrpcMessage(PlacementPolicy placementPolicy) {
        if (isNull(placementPolicy)) {
            return null;
        }

        var pp = Types.PlacementPolicy.newBuilder()
                .setUnique(placementPolicy.isUnique());

        for (Replica replica : placementPolicy.getReplicas()) {
            pp.addReplicas(ReplicaMapper.toGrpcMessage(replica));
        }

        return pp.build();
    }

    public static PlacementPolicy toModel(Types.PlacementPolicy placementPolicy) {
        if (isNull(placementPolicy) || placementPolicy.getSerializedSize() == 0) {
            return null;
        }

        return new PlacementPolicy(
                placementPolicy.getReplicasList().stream().map(ReplicaMapper::toModel).toArray(Replica[]::new),
                placementPolicy.getUnique()
        );
    }
}
