package info.frostfs.sdk.mappers.object;

import frostfs.object.Service;
import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectFilter;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;

import static info.frostfs.sdk.constants.ErrorConst.UNKNOWN_ENUM_VALUE_TEMPLATE;
import static java.util.Objects.isNull;

public class ObjectFilterMapper {
    private ObjectFilterMapper() {
    }

    public static Service.SearchRequest.Body.Filter toGrpcMessage(ObjectFilter<?> filter) {
        if (isNull(filter)) {
            return null;
        }

        var objectMatchType = Types.MatchType.forNumber(filter.getMatchType().value);
        if (isNull(objectMatchType)) {
            throw new ProcessFrostFSException(String.format(
                    UNKNOWN_ENUM_VALUE_TEMPLATE,
                    Types.MatchType.class.getName(),
                    filter.getMatchType().name()
            ));
        }

        return Service.SearchRequest.Body.Filter.newBuilder()
                .setMatchType(objectMatchType)
                .setKey(filter.getKey())
                .setValue(filter.getSerializedValue())
                .build();
    }
}
