package info.frostfs.sdk.mappers.response;

import frostfs.status.Types;
import info.frostfs.sdk.dto.response.ResponseStatus;
import info.frostfs.sdk.enums.StatusCode;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;

import static info.frostfs.sdk.constants.ErrorConst.UNKNOWN_ENUM_VALUE_TEMPLATE;
import static java.util.Objects.isNull;

public class ResponseStatusMapper {
    private ResponseStatusMapper() {
    }

    public static ResponseStatus toModel(Types.Status status) {
        if (isNull(status)) {
            return new ResponseStatus(StatusCode.SUCCESS);
        }

        var statusCode = StatusCode.get(status.getCode());
        if (isNull(statusCode)) {
            throw new ProcessFrostFSException(
                    String.format(UNKNOWN_ENUM_VALUE_TEMPLATE, StatusCode.class.getName(), status.getCode())
            );
        }

        return new ResponseStatus(statusCode, status.getMessage());
    }
}
