package info.frostfs.sdk.mappers.chain;

import com.google.protobuf.ByteString;
import frostfs.ape.Types;
import org.junit.jupiter.api.Test;

import java.util.Collections;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

public class ChainMapperTest {

    @Test
    void toModels_success() {
        //Given
        var chain1 = Types.Chain.newBuilder()
                .setRaw(ByteString.copyFrom(new byte[]{1, 2, 3, 4, 5}))
                .build();
        var chain2 = Types.Chain.newBuilder()
                .setRaw(ByteString.copyFrom(new byte[]{6, 7, 8, 9, 10}))
                .build();

        //When
        var result = ChainMapper.toModels(List.of(chain1, chain2));

        //Then
        assertNotNull(result);
        assertThat(result).hasSize(2);
        assertThat(result.get(0).getRaw()).containsOnly(chain1.getRaw().toByteArray());
        assertThat(result.get(1).getRaw()).containsOnly(chain2.getRaw().toByteArray());
    }

    @Test
    void toModels_null() {
        //When + Then
        assertNull(ChainMapper.toModels(null));
        assertNull(ChainMapper.toModels(Collections.emptyList()));
    }

    @Test
    void toModel_success() {
        //Given
        var chain = Types.Chain.newBuilder()
                .setRaw(ByteString.copyFrom(new byte[]{1, 2, 3, 4, 5}))
                .build();

        //When
        var result = ChainMapper.toModel(chain);

        //Then
        assertNotNull(result);
        assertThat(result.getRaw()).containsOnly(chain.getRaw().toByteArray());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(ChainMapper.toModel(null));
        assertNull(ChainMapper.toModel(Types.Chain.getDefaultInstance()));
    }
}
