package info.frostfs.sdk.mappers.object;

import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectAttribute;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

public class ObjectAttributeMapperTest {

    @Test
    void toGrpcMessages_success() {
        //Given
        var objectAttribute1 = new ObjectAttribute("key1", "value1");
        var objectAttribute2 = new ObjectAttribute("key2", "value2");

        //When
        var result = ObjectAttributeMapper.toGrpcMessages(Arrays.asList(objectAttribute1, objectAttribute2));

        //Then
        assertNotNull(result);
        assertThat(result).isNotNull().hasSize(2);
        assertEquals(objectAttribute1.getKey(), result.get(0).getKey());
        assertEquals(objectAttribute1.getValue(), result.get(0).getValue());
        assertEquals(objectAttribute2.getKey(), result.get(1).getKey());
        assertEquals(objectAttribute2.getValue(), result.get(1).getValue());
    }

    @Test
    void toGrpcMessages_null() {
        //When + Then
        assertNull(ObjectAttributeMapper.toGrpcMessages(null));
        assertNull(ObjectAttributeMapper.toGrpcMessages(Collections.emptyList()));
    }

    @Test
    void toGrpcMessage_success() {
        //Given
        var objectAttribute = new ObjectAttribute("key", "value");

        //When
        var result = ObjectAttributeMapper.toGrpcMessage(objectAttribute);

        //Then
        assertNotNull(result);
        assertEquals(objectAttribute.getKey(), result.getKey());
        assertEquals(objectAttribute.getValue(), result.getValue());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(ObjectAttributeMapper.toGrpcMessage(null));
    }

    @Test
    void toModel_success() {
        //Given
        var objectAttribute = Types.Header.Attribute.newBuilder()
                .setKey("key")
                .setValue("value")
                .build();

        //When
        var result = ObjectAttributeMapper.toModel(objectAttribute);

        //Then
        assertNotNull(result);
        assertEquals(objectAttribute.getKey(), result.getKey());
        assertEquals(objectAttribute.getValue(), result.getValue());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(ObjectAttributeMapper.toModel(null));
        assertNull(ObjectAttributeMapper.toModel(Types.Header.Attribute.getDefaultInstance()));
    }
}
