package info.frostfs.sdk.mappers.object;

import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectFilter;
import info.frostfs.sdk.enums.ObjectMatchType;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.mockito.MockedStatic;

import static info.frostfs.sdk.enums.ObjectMatchType.UNSPECIFIED;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.mockStatic;

public class ObjectFilterMapperTest {

    @ParameterizedTest
    @EnumSource(value = ObjectMatchType.class)
    void toGrpcMessage_success(ObjectMatchType type) {
        //Given
        var objectFilter = new ObjectFilter.FilterByAttribute(type, "key", "value");

        //When
        var result = ObjectFilterMapper.toGrpcMessage(objectFilter);

        //Then
        assertNotNull(result);
        assertEquals(objectFilter.getKey(), result.getKey());
        assertEquals(objectFilter.getValue(), result.getValue());
        assertEquals(objectFilter.getMatchType().value, result.getMatchType().getNumber());

    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(ObjectFilterMapper.toGrpcMessage(null));
    }

    @Test
    void toGrpcMessage_notValidType() {
        //Given
        var objectFilter = new ObjectFilter.FilterByAttribute(UNSPECIFIED, "key", "value");

        //When + Then
        try (MockedStatic<Types.MatchType> mockStatic = mockStatic(Types.MatchType.class)) {
            mockStatic.when(() -> Types.MatchType.forNumber(objectFilter.getMatchType().value))
                    .thenReturn(null);

            assertThrows(ProcessFrostFSException.class, () -> ObjectFilterMapper.toGrpcMessage(objectFilter));
        }
    }
}
