package info.frostfs.sdk.mappers.object;

import com.google.protobuf.ByteString;
import frostfs.refs.Types;
import info.frostfs.sdk.dto.object.ObjectId;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class ObjectIdMapperTest {

    @Test
    void toGrpcMessage_success() {
        //Given
        var objectId = new ObjectId("85orCLKSu3X1jGiTFmwmTUsBU88RBARNwuRwrEy5pyww");

        //When
        var result = ObjectIdMapper.toGrpcMessage(objectId);

        //Then
        assertNotNull(result);
        assertEquals(objectId.getValue(), new ObjectId(result.getValue().toByteArray()).getValue());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(ObjectIdMapper.toGrpcMessage(null));
    }

    @Test
    void toModel_success() {
        //Given
        var objectId = new ObjectId("85orCLKSu3X1jGiTFmwmTUsBU88RBARNwuRwrEy5pyww");
        var objectIdGrpc = Types.ObjectID.newBuilder()
                .setValue(ByteString.copyFrom(objectId.toHash()))
                .build();

        //When
        var result = ObjectIdMapper.toModel(objectIdGrpc);

        //Then
        assertNotNull(result);
        assertEquals(objectId.getValue(), result.getValue());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(ObjectIdMapper.toModel(null));
        assertNull(ObjectIdMapper.toModel(Types.ObjectID.getDefaultInstance()));
    }
}
