package info.frostfs.sdk.mappers.response;

import frostfs.status.Types;
import info.frostfs.sdk.enums.StatusCode;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;

import static org.junit.jupiter.api.Assertions.*;

public class ResponseStatusMapperTest {

    @ParameterizedTest
    @EnumSource(value = StatusCode.class)
    void toModel_success(StatusCode statusCode) {
        //Given
        var status = Types.Status.newBuilder()
                .setCode(statusCode.value)
                .setMessage("test")
                .build();

        //When
        var result = ResponseStatusMapper.toModel(status);

        //Then
        assertNotNull(result);
        assertEquals(status.getCode(), result.getCode().value);
        assertEquals(status.getMessage(), result.getMessage());
    }

    @Test
    void toModel_null() {
        //When
        var result = ResponseStatusMapper.toModel(null);

        //Then
        assertNotNull(result);
        assertEquals(StatusCode.SUCCESS.value, result.getCode().value);
        assertEquals("", result.getMessage());
    }

    @Test
    void toModel_notValid() {
        //Given
        var status = Types.Status.newBuilder()
                .setCode(-1)
                .setMessage("test")
                .build();

        //When + Then
        assertThrows(ProcessFrostFSException.class, () -> ResponseStatusMapper.toModel(status));
    }
}
